import React, { Component } from 'react';
// import { connect } from 'react-redux';
import * as Titles from './titles';
import DatePicker from 'react-native-datepicker';
import * as Formatter from './telephoneinput';
import * as Globals from './globals';
import * as Validation from './validation';
import CountryData from './country_data';
// import {
//     CardIOModule
// } from 'react-native-awesome-card-io';
import FastImage from 'react-native-fast-image';
import {
    Card,
    CardItem,
    Spinner,
    List,
    ListItem,
    Container,
    Header,
    ActionSheet,
    Title,
    Content,
    Footer,
    Input,
    FooterTab,
    Tab,
    TabHeading,
    Tabs,
    Button,
    Icon,
    Badge,
    Picker,
    CheckBox,
    Body,
    Left,
    Right,
    Text,
    InputGroup,
    H3,
    H2,
    H1,
} from 'native-base';
import Switch from './components/switch';
import HR from './components/hr';
import MapView from 'react-native-maps';
var { GooglePlacesAutocomplete } = require('react-native-google-places-autocomplete');

import * as UIA from './actions/uiActions';
// import * as Common from '../../customer/common';
import { View } from 'native-base';
import * as utils from './util';
import { Colors } from './colors';
import { Actions } from 'react-native-router-flux';

import {
    Image,
    StyleSheet, PixelRatio,
    Dimensions,
    InteractionManager,
    Platform,
    Slider,
    KeyboardAvoidingView,
    Animated,
    Alert,
} from 'react-native';
const { width, height } = Dimensions.get('window');



export const HeaderRight = (callback, value, title) => {
    return (
        <Right>
            {InputField({
                value: (value || (() => false)),
                title: title || '',
                type: 'switch',
                onValueChange: (callback || ((val) => {
                })),
                placeHolder: title || ''
            })}
        </Right>
    );
}

export const EmptyList = (emptyTitle) => {
    return (
        <ListItem>
            <Text>{emptyTitle || Titles.EmptyList}</Text>
        </ListItem>
    )
}
export const HeaderTitle = (text, style) => {
    style = style || {};
    return (
        <Body>
            <Title style={style}>
                {text || Titles.HeroCarCare}
            </Title>
        </Body>
    );
}


export const BOOLEAN_BUTTONS = (off, on) => {
    return [Titles.Cancel, on || Titles.On, off || Titles.Off];
}
export const BOOLEAN_BUTTONS_VALUE = (index) => {
    return BOOLEAN_BUTTONS()[index] === Titles.On;
}
export const displayMonth = (month) => {
    if (!isNaN(month))
        if (parseInt(month) < 10) {
            return '0' + month;
        }
    return month;
}

const useTabs = (ui) => {
    if (ui && ui.properties) {
        for (var i in ui.properties) {
            if (ui.properties[i] && ui.properties[i].length && ui.properties[i].some(t => !t.singular)) {
                return true;
            }
        }
    }
    return false;
}
const IsIOS = Platform.OS === 'ios';
const HEADER = '$ HEADER';
const SINGULAR = '$ singular';
const TITLE = '$ title';
const ICON = '$ icon';
const getTabSections = (ui) => {
    var result = [];
    var singular = {};
    var addedsingular = false;
    if (ui && ui.properties) {
        for (var i in ui.properties) {
            if (ui.properties[i] && ui.properties[i].length && ui.properties[i].some(t => !t.singular)) {
                var title = ui.properties[i].find(t => t.groupTitle) || {};
                var icon = ui.properties[i].find(t => t.groupIcon) || {}
                result.push({ [ICON]: icon.groupIcon, [HEADER]: i, [TITLE]: title.groupTitle, [i]: ui.properties[i] });
            }
            else if (ui.properties[i] && ui.properties[i].length) {
                singular = Object.assign(singular, { [TITLE]: Titles.General, [HEADER]: SINGULAR, [i]: ui.properties[i] });
                addedsingular = true;
            }
        }
        if (addedsingular) {
            result.unshift(singular);
        }
    }
    return result;
}

const getSingularSection = (ui) => {
    var result = [];
    var singular = {};
    var addedsingular = false;
    if (ui && ui.properties) {
        for (var i in ui.properties) {
            if (ui.properties[i] && ui.properties[i].length && ui.properties[i].some(t => t.singular)) {
                singular = Object.assign(singular, { [i]: ui.properties[i] });
                addedsingular = true;
            }
        }
        if (addedsingular) {
            result.push(singular);
        }
    }
    return result;
}

export const CommonUi = (ui, getState, setState, readonly, validation) => {
    if (ui) {
        var sections = null;
        if (useTabs(ui)) {
            sections = getTabSections(ui);
        }
        else {
            sections = getSingularSection(ui);
        }

        return getCommonUi(sections, getState, setState, readonly, validation);
    }
    return null;
}
const getCommonUi = (sections, getState, setState, readonly, validation) => {
    if (sections && sections.length > 1) {
        return (
            <Tabs>
                {sections.map((section, i) => getSection(section, i, getState, setState, readonly, validation))}
            </Tabs>
        )
    }
    else if (sections && sections.length) {
        return renderSection(sections[0], getState, setState, readonly, validation)
    }
    return null;
}
const getTabHeading = (section, i) => {
    return (
        <TabHeading key={'header' + i}>{section[ICON] ? <Icon name={section[ICON]} /> : null}{section[ICON] ? null : <Text>{section[TITLE] || section[HEADER]}</Text>}</TabHeading>
    );
};

const getSection = (section, i, getState, setState, readonly, validation) => {
    return (
        <Tab heading={getTabHeading(section, i)}>
            {renderSection(section, getState, setState, readonly, validation)}
        </Tab>
    );
}

const renderSection = (section, getState, setState, readonly, validation) => {
    var _sects = [];
    for (var i in section) {
        if (Array.isArray(section[i])) {
            section[i].map(t => t.$key = i);
            _sects = [..._sects, ...section[i]];
        }
    }

    return (
        <Card>
            <CardItem header>
                <Text>{section[TITLE]}</Text>
            </CardItem>
            {_sects.map(s => {
                return (
                    <CardItem key={'sects ' + s.$key + s.key}>
                        {_renderSection(s, getState, setState, readonly, validation)}
                    </CardItem>
                )
            })}</Card>
    );
}
const getValue = (state, path) => {
    var result;
    if (path === '') {
        return state;
    }
    if (state) {
        path.split('.').map((t, i) => {
            if (state && t) {
                state = state[t];
            }
        });
        result = state;
    }
    return result;
}
export const _commonReadonlyField = (opts) => {
    var { getValue, getState, value_path, title } = opts;
    return (
        <Left>
            <Body>
                <Text>{getValue(getState(), value_path) || ''}</Text>
                <Text note>{title}</Text>
            </Body>
        </Left>
    )
}
export const _renderSection = (section, getState, setState, readonly, validation) => {
    var sectionkey = section.key;
    var path = section.$key;
    sectionkey = sectionkey.lowercaseFirstLetter();
    var title = section.title || sectionkey;
    if (section.dependsOn) {
        if (!getValue(getState(), `${path}.${section.dependsOn.lowercaseFirstLetter()}`)) {
            return null;
        }
    }
    var value_path = !section.singular ? `${path}.${sectionkey}` : sectionkey;
    if (section.singular) {
        path = '';
    }
    switch (section.type) {
        case 'CHOICE':
            if (readonly) {
                return _commonReadonlyField({
                    getValue: (state, vp) => {
                        var value = getValue(getState(), value_path) || undefined;
                        return section.choices[value];
                    }, getState, value_path, title
                });
            }
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || undefined;
                },
                title: section.title,
                type: 'picker',
                onValueChange: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                    setState(path, update);
                },
                placeHolder: `${Titles.Select} ${section.title}`,
                items: () => {
                    var choiceResults = [];
                    for (var choice in section.choices) {
                        choiceResults.push((<Picker.Item key={'choice_' + choice} label={section.choices[choice]} value={choice} />));
                    }
                    return choiceResults;
                }
            });
            break;
        case 'ACCOUNTNUMBER':
            if (readonly) {
                return _commonReadonlyField({ getValue, getState, value_path, title });
            }
            return ValidatedInputField({
                validationrules: ['notempty', 'numbers', 'length9to17'],
                value_path,
                path,
                getState,
                sectionkey,
                setState,
                readonly,
                title,
                validation
            });

            break;
        case 'STATE':
            if (readonly) {
                return _commonReadonlyField({
                    getValue: (state, vp) => {
                        var value = getValue(getState(), value_path) || undefined;
                        var cod = CountryData.states.find(d => d.abbreviation == value)
                        if (cod) {
                            return cod.name;
                        }
                        return Titles.SelectState;
                    }, getState, value_path, title
                });
            }
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || undefined;
                },
                title: Titles.StateProvince,
                type: 'picker',
                onValueChange: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                    setState(path, update);
                },
                placeHolder: Titles.SelectState,
                items: () => {
                    return CountryData.states.map(d => {
                        return (
                            <Picker.Item key={d.abbreviation} label={d.name} value={d.abbreviation} />
                        );
                    })
                }
            })
            break;
        case 'COUNTRY':
            if (readonly) {
                return _commonReadonlyField({
                    getValue: (state, vp) => {
                        var value = getValue(getState(), value_path) || undefined;
                        var cod = CountryData.allCountriesObject.find(d => d.ISO2.toUpperCase() == value)
                        if (cod) {
                            return cod.name;
                        }
                        return Titles.SelectCountry;
                    }, getState, value_path, title
                });
            }
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || undefined;
                },
                title: Titles.Country,
                type: 'picker',
                onValueChange: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                    setState(path, update);
                },
                placeHolder: Titles.SelectCountry,
                items: () => {
                    return CountryData.allCountriesObject.map(d => {
                        return (
                            <Picker.Item key={d.name} label={d.name} value={d.ISO2.toUpperCase()} />
                        );
                    })
                }
            });
            break;
        case 'CURRENCY':
            if (readonly) {
                return _commonReadonlyField({
                    getValue: (state, vp) => {
                        var value = getValue(getState(), value_path) || undefined;
                        var cod = value;
                        if (cod) {
                            return value;
                        }
                        return Titles.SelectCurrency;
                    }, getState, value_path, title
                });
            }
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || undefined;
                },
                title: Titles.Currency,
                type: 'picker',
                onValueChange: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                    setState(path, update);
                },
                placeHolder: Titles.SelectCurrency,
                items: () => {
                    return CountryData.currencies.map(d => {
                        return (
                            <Picker.Item key={d} label={d} value={d} />
                        );
                    })
                }
            });
            break;
        case 'ROUTINGNUMBER':
            if (readonly) {
                return (
                    <Left>
                        <Body>
                            <Text>{getValue(getState(), value_path) || ''}</Text>
                            <Text note>{title}</Text>
                        </Body>
                    </Left>
                )
            }
            return ValidatedInputField({
                validationrules: ['notempty', 'numbers', 'length9'],
                value_path,
                path,
                getState,
                sectionkey,
                setState,
                readonly,
                title,
                validation
            });
            break;
        case 'GEOLOCATION':
            var coordinate = getValue(getState(), value_path);
            return (
                InputField({
                    type: 'loc_rad',
                    value: () => getValue(getState(), value_path),
                    title: section.title || Titles.Location,
                    viewProps: () => { return { flex: 1 } },
                    onValueChange: (val) => {
                        // me.onValueChange('point', val);
                        if (!readonly && coordinate && coordinate.latitude === 0 && coordinate.longitude === 0) {
                            var update = Object.assign({}, getValue(getState(), path), {
                                [sectionkey]: val
                            });
                            setState(path, update);
                        }
                    },
                    extra: () => {
                        return (
                            //[
                            coordinate ? (<MapView.Marker draggable
                                coordinate={coordinate}
                                onDragEnd={(e) => {
                                    if (!readonly) {
                                        var update = Object.assign({}, getValue(getState(), path), {
                                            [sectionkey]: e.nativeEvent.coordinate
                                        });
                                        setState(path, update);
                                    }
                                }}
                            />) : null
                            // , <MapView.Circle
                            //     key={(promoCode.radius) + JSON.stringify(promoCode.point)}
                            //     fillColor={Colors.HeroBlue + '55'}
                            //     center={promoCode.point}
                            //     radius={1000 * ((promoCode.radius) || .1)} />
                            //]
                        )
                    },
                    placeHolder: section.title || Titles.Location
                })
            )
            break;
        case 'ADDRESS':
            var googlePlacesApi = Globals.getGooglePlacesApi();
            if (readonly || !googlePlacesApi) {
                return _commonReadonlyField({
                    getValue: (state, vp) => {
                        var value = getValue(getState(), value_path) || undefined;
                        if (value) {
                            return value.formatted_address;
                        }
                        return Titles.SelectAddress;
                    }, getState, value_path, title
                });
            }

            return (
                InputField({
                    // errorIcon: () => { return (field.icon ? <Icon name={field.icon} /> : null); },
                    getAddress: () => GetAddressInput({
                        value_path, title, googlePlacesApi, getValue, getState, path, sectionkey, setState
                    }),
                    type: 'address',
                    //    viewProps: () => { return { key: field.key || ('field.name' + field.name) } },
                    value: () => getValue(getState(), value_path) || undefined,
                    title: title,
                    inputProps: () => { return { autoCapitalize: 'none' } },
                    //onChangeText: field.onValueChange.bind(me, field.name),
                    placeHolder: title
                })
            );
            break;
        case 'LONGSTRING':
            if (readonly) {
                return (
                    <Left>
                        <Body>
                            <Text>{getValue(getState(), value_path) || ''}</Text>
                            <Text note>{title}</Text>
                        </Body>
                    </Left>
                )
            }
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || '';
                },
                onChangeText: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                    setState(path, update);
                },
                title: title,
                type: 'autotext',
                editable: () => !readonly,
                placeHolder: title
            });
        case 'STRING':
            if (readonly) {
                return (
                    <Left>
                        <Body>
                            <Text>{getValue(getState(), value_path) || ''}</Text>
                            <Text note>{title}</Text>
                        </Body>
                    </Left>
                )
            }
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || '';
                },
                onChangeText: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                    setState(path, update);
                },
                title: title,
                type: 'string',
                editable: () => true,
                placeHolder: title
            });
        case 'MONEY':
            if (readonly) {
                var val = getValue(getState(), value_path) || '';
                return (
                    <Left>
                        <Body>
                            <Text>{Formatter.default.currency(val.toString())}</Text>
                            <Text note>{title}</Text>
                        </Body>
                    </Left>
                )
            }
            return InputField({
                value: () => {
                    var val = getValue(getState(), value_path) || '';
                    return Formatter.default.currency(val.toString());
                },
                onChangeText: (value) => {
                    var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: parseFloat(Formatter.default.removeNonNumbers(value)) });
                    setState(path, update);
                },
                title: title,
                inputProps: () => { return { keyboardType: "numeric" } },
                type: 'string',
                editable: () => true,
                placeHolder: title
            });
        case 'BOOLEAN':
            return InputField({
                value: () => {
                    return getValue(getState(), value_path) || false;
                },
                onValueChange: (value) => {
                    if (!readonly) {
                        var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
                        setState(path, update);
                    }
                },
                title: title,
                type: 'switch',
                editable: () => !readonly,
                placeHolder: title
            });
    }
}

export const GetAddressInput = (opts) => {
    var { value_path, title, googlePlacesApi, getValue, getState, path, sectionkey, setState } = opts;
    return (
        <GooglePlacesAutocomplete
            placeholder={title}
            minLength={2}
            autoFocus={false}
            listViewDisplayed='auto'
            fetchDetails={true}
            renderDescription={(row) => {
                if (row && row.terms && row.terms.length) {
                    return row.terms.map(x => x.value).join(', ');
                }
                return null;
            }}
            onPress={(data, details = null) => {
                var update = { addressComponents: details.address_components.map(x => x) };
                if (details && details.geometry && details.geometry.location) {
                    var loc = details.geometry.location;
                    update = Object.assign({}, update, {
                        latitude: loc.lat,
                        longitude: loc.lng
                    });
                }
                if (details && details.geometry && details.geometry.viewport) {
                    var viewport = details.geometry.viewport;
                    if (viewport.northeast && viewport.southwest) {
                        update = Object.assign({}, update,
                            {
                                latitudeDelta: viewport.northeast.lat - viewport.southwest.lat,
                                longitudeDelta: viewport.northeast.lng - viewport.southwest.lng,
                            });
                    }
                }
                else {
                    update = Object.assign({}, update,
                        {
                            latitudeDelta: 0.015,
                            longitudeDelta: 0.0121,
                        })
                }
                if (update) {
                    update = Object.assign(update, { formatted_address: details.formatted_address });
                }

                var _update = Object.assign({}, getValue(getState(), path), { [sectionkey]: update });
                setState(path, _update);
            }}
            getDefaultValue={() => {
                var value = getValue(getState(), value_path) || undefined;
                if (value) {
                    return value.formatted_address;
                }
                return ''; // text input default value
            }}
            query={{
                key: googlePlacesApi,
                language: 'en', // language of the results
            }}
            enablePoweredByContainer={false}
            styles={{
                description: {
                    fontWeight: 'bold',
                },
                listView: {
                    backgroundColor: 'transparent',
                },

                predefinedPlacesDescription: {

                },
                textInput: {
                    backgroundColor: 'transparent',
                },
                textInputContainer: {
                    borderLeftWidth: 0,
                    borderRightWidth: 0,
                    backgroundColor: 'transparent',
                    height: 44,
                    borderTopColor: 'transparent',
                    borderBottomColor: 'transparent',
                }
            }}
            currentLocationLabel={Titles.CurrentLocation}
            nearbyPlacesAPI='GooglePlacesSearch'
            GoogleReverseGeocodingQuery={{
            }}

            filterReverseGeocodingByTypes={['locality', 'administrative_area_level_3']}
        // filter  
        />
    )
}

export const BackButton = (ops) => {
    ops = Object.assign({ action: () => { }, style: {} }, ops)
    return (

        <Button transparent onPress={() => {
            ops.action();
            if (!ops.cancel)
                Actions.pop();
        }}>
            <Icon name='md-arrow-round-back' style={ops.style} />
        </Button>
    );
}

export const ValidatedInputField = (opts) => {
    var { title, validationrules, value_path, path, getState, sectionkey, setState, readonly, validation } = opts;
    return InputField({
        value: () => {
            return getValue(getState(), value_path) || '';
        },
        onChangeText: (value) => {
            var update = Object.assign({}, getValue(getState(), path), { [sectionkey]: value });
            var updatedState = setState(path, update);
            var validationResult = Validation.validate({
                [value_path]: Validation.validation(validationrules)
            }, updatedState, getValue);
            if (validationResult) {
                validation.set(null, validationResult.validationState)
            }
        },
        title: title,
        type: 'string',
        editable: () => true,
        placeHolder: title
    }, Validation.InputField(getValue(validation.get(), value_path)));
}
export const ListButtonItem = (icon, title, action, extras, right, key) => {
    var _right = null;
    var _extras = null;
    if (extras) {
        extras = <Body>{extras}</Body>
    }
    if (right) {
        _right = <Right>{right}</Right>
    }
    return (
        <ListItem style={{
            marginLeft: 0,
            paddingLeft: 15
        }} button key={key} onPress={action}>
            <Left>
                <Icon name={icon} />
                <Text>{title}</Text>
            </Left>
            {_extras}
            {_right}
        </ListItem>
    )
}

export const ErrorText = (text) => {
    return (
        <Text style={{ fontSize: 12, color: Colors.Danger }}>{text}</Text>
    );
}

export const InputProps = (value, onChangeText, placeHolderText, label, keyboardType) => {
    return {
        value: value,
        keyboardType: keyboardType || 'default',
        onChangeText: onChangeText,
        label: label || '',
        placeholder: placeHolderText
    }
}
export const runAfter = (me, callback) => {
    me.setState({ busy: true })
    InteractionManager.runAfterInteractions(() => {
        if (callback)
            callback();
        me.setState({ busy: false });
    });
}

export const whileNotBusy = (callback) => {
    return new Promise((resolve, fail) => {
        InteractionManager.runAfterInteractions(() => {
            try {
                if (callback)
                    callback();
                resolve();
            } catch (e) {
                fail(e);
            }
        });
    });
}


export const getStreetAddress = addressComponents => {
    return getPartOfComponents(['street_number', 'route'], addressComponents);
};
export const getCity = addressComponents => {
    return getPartOfComponents(['locality'], addressComponents);
}
export const getStateOrProvince = addressComponents => {
    return getPartOfComponents(["administrative_area_level_1"], addressComponents);
}
export const getZipCode = addressComponents => {
    return getPartOfComponents(["postal_code"], addressComponents);
}
const getPartOfComponents = (types, addressComponents) => {
    return addressComponents ? addressComponents.filter(x =>
        types.find(t => x.types.indexOf(t) !== -1))
        .map(t => t.longName)
        .join(' ') : null;
}
export const Circle = (children, style) => {
    return (
        <View style={style}>
            <View
                style={{
                    flex: 1,
                    alignItems: 'center',
                    flexDirection: 'column'
                }}>
                <View style={{
                    flex: 1,
                    alignItems: 'center',
                    flexDirection: 'row'
                }}>
                    {children}
                </View>
            </View>
        </View>
    )
}
export const ProcessCircle = (children, style) => {
    return Circle(children, Object.assign({
        height: 19,
        width: 19,
        borderRadius: 19,
        backgroundColor: Colors.HeroBlue
    }, style));
}
export const DblCircle = (child, rad, rad2) => {
    return (
        Circle(
            Circle(child, {
                height: rad - rad2,
                width: rad - rad2,
                borderRadius: rad - rad2,
                backgroundColor: Colors.White
            }), {
                height: rad,
                borderRadius: rad,
                width: rad,
                backgroundColor: Colors.HeroBlue
            })
    )
}
export const dateTime = (datetimestring) => {
    var date = new Date(datetimestring);
    if (isNaN(date.getTime())) {
        return '';
    }
    return date.toLocaleDateString() + ' ' + date.toLocaleTimeString();
}
export const localDate = (datetimestring) => {
    var date = new Date(datetimestring);
    if (isNaN(date.getTime())) {
        return '';
    }
    return date.toLocaleDateString();
}

export function timeSinceDate(dateString) {
    var date = new Date(dateString);
    return timeSince(date);
}

export function timeSince(date) {
    if (!date.getTime || isNaN(date.getTime())) {
        return '';
    }
    var seconds = Math.floor((new Date() - date) / 1000);

    var interval = Math.floor(seconds / 31536000);

    if (interval > 1) {
        return date.toLocaleDateString();
    }
    interval = Math.floor(seconds / 2592000);
    if (interval > 1) {
        return interval + Titles.Ago(" months");
    }
    interval = Math.floor(seconds / 86400);
    if (interval > 1) {
        return interval + Titles.Ago(" days");
    }
    interval = Math.floor(seconds / 3600);
    if (interval > 1) {
        return interval + Titles.Ago(" hours");
    }
    interval = Math.floor(seconds / 60);
    if (interval > 1) {
        return interval + Titles.Ago(" minutes");
    }
    return Math.floor(seconds) + " seconds";
}

export const writeLocation = point => {
    if (point && point.latitude && point.longitude) {
        return point.latitude + '°, ' + point.longitude + '°';
    }
    return '';
}
export const InformationCardItem = (text, note, icon, key) => {
    icon = icon || null;
    if (typeof (icon) == 'string') {
        icon = <Icon name={icon} />;
    }
    var extras = {};
    if (key) {
        extras = { key };
    }
    return (
        <CardItem  {...extras} >
            {icon}
            <Body>
                <Text>{text}</Text>
                <Text note>{note}</Text>
            </Body>
        </CardItem>
    );
}

export const sortByCreated = (asc) => {
    if (asc)
        return (a, b) => { return Date.parse(a.created) - Date.parse(b.created) };
    return (b, a) => { return Date.parse(a.created) - Date.parse(b.created) };
}

export const getText = (key, obj) => {
    if (obj && obj[key] && obj[key].toFixed)
        return obj[key].toFixed(2);
    if (obj && obj[key]) {
        return obj[key];
    }
    return '0.00';
}
export const percentage = num => {
    if (isNaN(num)) {
        return '';
    }
    return (Math.floor(num * 100));
}

export const flex = () => {
    return { flex: 1 };
}

export const image = (profileImage, imagesize) => {
    var pimage = profileImage ? profileImage.replace('sz=50', `sz=${imagesize}`) : '';
    if (pimage.indexOf('facebook') !== -1) {
        pimage += "?type=large"
    }
    return pimage;
}

export const profileImage = (profileImage, imagesize, icon) => {
    if (!profileImage) {
        icon = icon || 'md-person';
        return (<Icon name={icon} style={{ color: Colors.Laborer }} />)
    }
    imagesize = imagesize || 41;
    var pimage = image(profileImage, imagesize);
    return (
        <View>
            {pimage ? <FastImage style={{
                width: imagesize,
                height: imagesize,
                borderRadius: imagesize / 2,
            }} source={{ uri: pimage }} /> : null}
        </View>
    );
}

export const profileImg = (profileImage, imagesize, borderRadius, style) => {
    if (!profileImage) {
        return (<Icon name='md-person' style={{ color: Colors.Laborer }} />)
    }
    style = style || {};
    imagesize = imagesize || 41;
    var pimage = image(profileImage, imagesize);
    return (
        pimage ? <FastImage style={Object.assign({
            width: imagesize,
            height: imagesize,
            borderRadius: borderRadius || imagesize / 2,
        }, style)} source={{ uri: pimage }} /> : null
    );
}

export const ErrorStateProp = name => {
    return name + '_$_error';
}

export const InputGroupView = () => {
    return {
        flex: 1,
        flexDirection: 'column',
        borderBottomWidth: 1,
        borderColor: TextColor() + '44'
    }
}
export const PickerText = (dark) => {
    return { color: TextColor(dark), fontSize: 13, lineHeight: 16 };
}


export const TextColor = (dark) => {
    return Colors.Dark;
}
export const InputField = (options, errorStuff) => {
    options = options || {};
    options = Object.assign({}, errorStuff, options);
    options = Object.assign({
        relativePosStyle: InputGroupView(),
        titleStyle: Object.assign(PickerText(options.lightMode), flex(), options.required && options.required() ? { fontWeight: '900' } : {}, {}),
        title: '',
        value: () => { },
        onChangeText: () => { },
        onchange: () => { },
        noHR: () => true,
        errorIcon: () => { return null },
        placeHolder: '',
        inputStyle: () => {
            return {};
        },
        required: () => false,
        inputProps: () => { return {}; },
        errorGroupState: () => { { } },
        onBlur: () => { },
        editable: () => true,
        errorText: () => { },
        style: () => { return {}; },
        lightMode: false,
        min: () => 0,
        max: () => 1,
        step: () => 0,
        hrOptions: () => { return {} },
        getAddress: () => { return null },
        viewProps: () => { return {} },
        extra: () => { return null },
        onSubmitEditing: () => {
            return () => { }
        }
    }, options)
    var inputprops = Object.assign({
        onSubmitEditing: options.onSubmitEditing(),
        onBlur: options.onBlur(),
        autoCapitalize: 'none'
    }, options.inputProps());
    var input_field = null;
    var hr_ = (options.lightMode && !options.noHR()) ? <HR lineColor={Colors.Dark} {...options.hrOptions() } /> : null;
    var alwaysshowtitle = false;
    if (options.type === 'address') {
        input_field = (
            <View style={{ flexDirection: 'row' }}>
                {options.errorIcon()}
                {options.getAddress()}
            </View>
        );
    }
    else if (options.type === 'switch') {
        alwaysshowtitle = true;
        input_field = (
            <View style={{ flexDirection: 'column' }}>
                <View style={{ flexDirection: 'row' }}>
                    <Right>
                        <Switch value={options.value()}
                            disabled={!options.editable()}
                            onValueChange={options.onValueChange} />
                    </Right>
                </View>
            </View>
        );
    }
    else if (options.type === 'check') {
        input_field = (
            <View style={{ flexDirection: 'column' }}>
                <ListItem onPress={() => {
                    if (options.onValueChange && options.editable())
                        options.onValueChange(!options.value());
                }}>
                    <Right>
                        <CheckBox
                            onPress={() => {
                                if (options.onValueChange && options.editable())
                                    options.onValueChange(!options.value());
                            }}
                            checked={options.value()}
                            disabled={!options.editable()}
                            onValueChange={options.onValueChange} />
                    </Right>
                </ListItem>
                {hr_}
            </View>
        );
    }
    else if (options.type === 'actionsheet') {
        var action_sheet_options = options.destructiveButtonIndex ? {
            destructiveButtonIndex: options.destructiveButtonIndex()
        } : {};
        input_field = (
            <Button onPress={() => ActionSheet.show(
                Object.assign({
                    options: options.buttons(),
                    cancelButtonIndex: options.cancelindex(),
                    title: options.title
                }, action_sheet_options),
                (buttonIndex) => options.onValueChange(buttonIndex)
            )}><Text>{options.title}</Text></Button>
        );
    }
    else if (options.type === 'slider') {
        alwaysshowtitle = true;
        input_field = (
            <View style={{ flexDirection: 'column' }}>
                <View style={{ flexDirection: 'row' }}>
                    {options.errorIcon()}
                    <Slider
                        style={{ flex: 3 }}
                        step={options.step()}
                        maximumValue={options.max()}
                        minimumValue={options.min()}
                        value={options.value()}
                        disabled={!options.editable()}
                        onSlidingComplete={options.onValueChange}
                        onValueChange={options.onchange} />
                    {options.extra()}
                </View>
                {hr_}
            </View>
        )
    }
    else if (options.type === 'loc_rad') {
        var map_options = {};
        if (options.width) {
            map_options = { width: options.width() };
        }
        input_field = (
            <View style={{ flexDirection: 'column', backgroundColor: 'red' }}>
                <View style={{ flexDirection: 'row', backgroundColor: 'red' }}>
                    {options.errorIcon()}
                    <MapView
                        style={Object.assign({ flex: 1, height: 300, backgroundColor: 'blue', })}
                        onRegionChange={options.onValueChange}
                        initialRegion={options.value()}  >
                        {options.extra()}
                    </MapView>
                </View>
                {hr_}
            </View>
        )
    }
    else if (options.type === 'date') {
        input_field = (
            <View style={{ flexDirection: 'column' }}>
                <View style={{ flexDirection: 'row' }}>
                    {options.errorIcon()}
                    <DatePicker
                        date={options.value() || new Date()}
                        mode="date"
                        placeholder={''}
                        format="YYYY-MM-DD"
                        disabled={!options.editable()}
                        confirmBtnText="Confirm"
                        cancelBtnText="Cancel"
                        customStyles={{
                            dateIcon: {
                                position: 'absolute',
                                left: 0,
                                top: 4,
                                marginLeft: 0,
                                height: 0,
                                width: 0
                            },
                            dateText: {
                                color: TextColor(),
                            },
                            dateInput: {
                                borderWidth: 0,
                            },
                            disabled: {
                                backgroundColor: 'transparent',
                            }
                        }}
                        onDateChange={options.onValueChange}
                    />
                </View>
                {hr_}
            </View>
        );
    }
    else if (options.type === 'picker') {
        var pickerops = {};//{theme: PickerTheme(), textStyle: PickerText(options.lightMode) };
        input_field = (
            <View style={{ flexDirection: 'column' }}>
                <View style={{ flexDirection: 'row' }}>
                    {options.errorIcon()}
                    <Picker
                        style={flex()}
                        placeholder={options.placeHolder}
                        {...pickerops}
                        selectedValue={options.value()}
                        onValueChange={options.onValueChange}>
                        {options.items()}
                    </Picker>
                </View>
                {hr_}
            </View>
        );
    }

    else if (options.type === 'autotext') {
        input_field = (
            <KeyboardAvoidingView behavior={'position'}>
                <View style={{ flexDirection: 'column' }}>
                    <InputGroup  {...options.errorGroupState() }>
                        {options.errorIcon()}
                        <TextInput
                            autoGrow={true}
                            {...inputprops}
                            style={Object.assign(flex(), {
                                paddingLeft: 10,
                                fontSize: 17,
                                flex: 1,
                                backgroundColor: 'white',
                                borderWidth: 0,
                                borderRadius: IsIOS ? 4 : 0,
                            }, options.lightMode ? { color: Colors.Dark } : null, options.inputStyle())}
                            value={options.value()}
                            maxHeight={200}
                            onChangeText={options.onChangeText}
                            placeholder={options.placeHolder} />

                    </InputGroup>
                    {hr_}
                    {options.errorText()}
                </View>
            </KeyboardAvoidingView>
        );
    }
    else {
        input_field = (
            <KeyboardAvoidingView behavior={'position'}>
                <View style={{ flexDirection: 'column' }}>
                    <InputGroup  {...options.errorGroupState() }>
                        {options.errorIcon()}
                        <Input
                            {...inputprops}
                            style={Object.assign(flex(), options.lightMode ? { color: Colors.Dark } : null, options.inputStyle())}
                            value={options.value()}
                            onChangeText={options.onChangeText}
                            placeholderTextColor={options.required() ? Colors.Danger : Colors.LighterDark}
                            placeholder={options.placeHolder + (options.required() ? '*' : '')} />

                    </InputGroup>
                    {hr_}
                    {options.errorText()}
                </View>
            </KeyboardAvoidingView>
        );
    }
    return (
        <View {...options.viewProps() } style={options.relativePosStyle}>
            <Text style={options.titleStyle}>
                {options.value() || alwaysshowtitle ? options.title : ''} {options.value() && options.required() ? '*' : ''}
            </Text>
            {input_field}
        </View>
    )
}

export const WaitingFooter = (me) => {
    return (
        <Footer>
            {SpinnerFooter()}
        </Footer>
    );
}


export const ErrorProps = (state, key, dark) => {
    if (key === undefined) {
        return state ? { style: Object.assign({ color: Colors.Danger }, ListIconStyle(dark)) } : { style: ListIconStyle(dark) }
    }
    else {
        var error = UIA.Get(state, key) ? { style: Object.assign({ color: Colors.Danger }, ListIconStyle(dark)) } : { style: ListIconStyle(dark) };
        return error;
    }

}


export const MediaListItems = (me, resource, candelete, opts) => {
    var name_icon = null;
    var pimage = image(resource.url);
    opts = Object.assign({ onPress: () => { }, hideDate: false, imagesize: 30 }, opts);
    var { imagesize } = opts;
    if (resource.type === 'IMAGE') {
        name_icon = (
            <FastImage style={{
                width: imagesize,
                height: imagesize,
            }} source={{ uri: pimage }} />
        );
    }
    return (
        <ListItem key={resource.id} onPress={() => {
            opts.onPress();
        }}>
            <Left>{name_icon}
                {opts.hideDate ? null : <Text note>{dateTime(resource.created)}</Text>}
            </Left>
            {candelete ? <Right>
                <Button transparent onPress={() => {
                    me.props.deleteResource(resource.id);
                }}>
                    <Icon name="md-close" />
                </Button>
            </Right> : null}
        </ListItem>
    );
};
export const MediaViewItem = (me, resource, candelete, opts) => {
    var name_icon = null;
    opts = Object.assign({ hideDate: true, imagesize: 30 }, opts);
    var { imagesize } = opts;
    var pimage = image(resource.url);
    if (resource.type === 'IMAGE') {
        name_icon = (
            <FastImage style={{
                width: imagesize,
                height: imagesize,
            }} source={{ uri: pimage }} />
        );
    }
    return (
        <View key={resource.id}>
            <Left>{name_icon}
                {opts.hideDate ? null : <Text>{dateTime(resource.created)}</Text>}
            </Left>
            {candelete ? <Right>
                <Button transparent onPress={() => {
                    me.props.deleteResource(resource.id);
                }}>
                    <Icon name="md-close" />
                </Button>
            </Right> : null}
        </View>
    );
};


export function SpinnerFooter(style) {
    return (
        <View style={Object.assign({}, style)}>
            <Spinner size="small" color={TextColor()} />
        </View>
    );
}

export const percentageFormatted = (val) => {
    val = Math.round(val * 100);
    return (val).toFixed(2).toString();
}
export const convertToCurrency = (val) => {
    if (isNaN(val)) {
        val = 0;
    }
    val = Math.round(val);
    return (val / 100).toFixed(2).toString();
}

export const showConvertMeterToMile = val => {
    var res = 0;

    if (!res) {
        res = Math.round(1000 * (val)) / 1000;
    }

    return res;
}
const mtom = 0.000621371;
const metomi = 1609.344;
export const convertMeterToMile = val => {
    return val / metomi;
}
export const convertPriceMeterToMile = val => {
    return val / mtom;
}

export const _getDistance = (directions) => {
    var result = Titles.Unknown;
    if (directions && directions.routes) {
        var distances = directions.routes.map(route => {
            var dist = 0;
            if (route.legs) {
                route.legs.forEach(leg => {
                    if (leg.distance && leg.distance.value) {
                        dist += leg.distance.value;
                    }
                });
            }
            return dist;
        });
        if (distances.length) {
            result = distances[0];
        }
    }
    return result;
}

export const SomethingWentWrong = (callback, alternateError) => {
    return () => {
        Alert.alert(Titles.Error, alternateError || Titles.SomethingWentWrong, [{
            text: Titles.Ok, onPress: () => {
                if (callback) {
                    callback();
                }
            }
        }])
    };
}
export const Ok = (title, message, callback) => {
    return () => {
        Alert.alert(title, message, [{
            text: Titles.Ok, onPress: callback || (() => { })
        }])
    };
}

export const OkCancel = (title, message, callback, cancel) => {
    return () => {
        Alert.alert(title, message, [{
            text: Titles.Ok, onPress: callback || (() => { })
        }, { text: Titles.Cancel, onPress: cancel || (() => { }) }])
    };
}

export const GetAddressSelect = (me, field) => {
    var api = me.state.apiKey;
    if (!api) {
        return null;
    }
    if (field.onload && !me.state.loaded) {
        return null;
    }
    return (
        <GooglePlacesAutocomplete
            key={field.name + '#field'}
            placeholder={field.placeholder}
            minLength={2}
            autoFocus={false}
            listViewDisplayed='auto'
            fetchDetails={true}
            renderDescription={(row) => {
                if (row && row.terms && row.terms.length) {
                    return row.terms.map(x => x.value).join(', ');
                }
                return null;
            }}
            onPress={(data, details = null) => {
                var update = { addressComponents: details.address_components.map(x => x) };
                if (details && details.geometry && details.geometry.location) {
                    var loc = details.geometry.location;
                    update = Object.assign({}, update, {
                        latitude: loc.lat,
                        longitude: loc.lng
                    });
                }
                if (details && details.geometry && details.geometry.viewport) {
                    var viewport = details.geometry.viewport;
                    if (viewport.northeast && viewport.southwest) {
                        update = Object.assign({}, update,
                            {
                                latitudeDelta: viewport.northeast.lat - viewport.southwest.lat,
                                longitudeDelta: viewport.northeast.lng - viewport.southwest.lng,
                            });
                    }
                }
                else {
                    update = Object.assign({}, update,
                        {
                            latitudeDelta: 0.015,
                            longitudeDelta: 0.0121,
                        })
                }
                if (update) {
                    update = Object.assign(update, { formatted_address: details.formatted_address });
                    if (me.mounted)
                        me.setState({ updated: true, [field.name]: update });
                }
            }}
            getDefaultValue={() => {
                return me.state[field.name] || ''; // text input default value
            }}
            query={{
                key: api,
                language: 'en', // language of the results

            }}
            enablePoweredByContainer={false}
            styles={{
                description: {
                    fontWeight: 'bold',
                },
                listView: {
                    backgroundColor: 'transparent',
                },

                predefinedPlacesDescription: {

                },
                textInput: {

                    backgroundColor: 'transparent',
                },
                textInputContainer: {
                    borderLeftWidth: 0,
                    borderRightWidth: 0,
                    backgroundColor: 'transparent',
                    height: 44,
                    borderTopColor: 'transparent',
                    borderBottomColor: 'transparent',
                }
            }}
            currentLocationLabel={Titles.CurrentLocation}
            nearbyPlacesAPI='GooglePlacesSearch'
            GoogleReverseGeocodingQuery={{
            }}

            filterReverseGeocodingByTypes={['locality', 'administrative_area_level_3']}
        // filter  
        />
    );
}
export const Waiting = () => {
    return (
        <View style={{
            flex: 1,
            alignItems: 'center',
            justifyContent: 'center',
            backgroundColor: 'black'
        }}>
            <View style={{
                flex: 1,
                alignItems: 'center',
                justifyContent: 'center',
                flexDirection: 'column'
            }}>
                {Banner({})}
                <Spinner color={Colors.White} />
            </View>
        </View>
    );
}
var CardIOModule = null;
export const setScanCardService = (cardIOModule) => {
    CardIOModule = cardIOModule;
}
export const scanCard = (callback, error) => {
    error = error || (() => { });
    callback = callback || (function () { });
    CardIOModule
        .scanCard({
            hideCardIOLogo: true,
            requireCardholderName: true
        })
        .then(callback)
        .catch(error)
}
export const Banner = (style, logo) => {
    style = style || {};
    return (
        <View style={Object.assign({}, {
            flex: 1,
            flexDirection: 'column',
            justifyContent: 'center',
            alignItems: 'center',
        }, style)}>
            <Image style={{
                resizeMode: 'contain',
                height: 100,
                width: Math.max(100, width - 200)
            }} source={logo} />
        </View>
    );
}

export const CreateFields = (me, fields) => {
    return (fields || []).map(field => {
        // if (field.type === 'address') {
        //     return GetAddressSelect(me, field);
        // }
        return (
            InputField({
                errorIcon: () => { return (field.icon ? <Icon name={field.icon} /> : null); },
                getAddress: () => { return GetAddressSelect(me, field) },
                type: field.type,
                viewProps: () => { return { key: field.key || ('field.name' + field.name) } },
                value: () => me.state[field.name] || '',
                title: field.placeholder,
                inputProps: () => { return { autoCapitalize: 'none' } },
                onChangeText: field.onValueChange.bind(me, field.name),
                placeHolder: field.placeholder
            })
        );
    });
}
